package io.lumine.cosmetics.menus;

import io.lumine.cosmetics.MCCosmeticsPlugin;
import io.lumine.cosmetics.api.menus.SelectionMenuContext;
import io.lumine.cosmetics.commands.CommandHelper;
import io.lumine.cosmetics.config.Scope;
import io.lumine.cosmetics.players.Profile;
import io.lumine.cosmetics.types.MCCosmeticsManager;
import io.lumine.mythic.bukkit.utils.Schedulers;
import io.lumine.mythic.bukkit.utils.config.properties.Property;
import io.lumine.mythic.bukkit.utils.config.properties.types.MenuProp;
import io.lumine.mythic.bukkit.utils.menu.EditableMenuBuilder;
import io.lumine.mythic.bukkit.utils.menu.Icon;
import io.lumine.mythic.bukkit.utils.menu.MenuHelper;
import io.lumine.mythic.bukkit.utils.prompts.chat.ChatPrompt;
import org.bukkit.entity.Player;

public class SelectionMenu extends CosmeticMenu<SelectionMenuContext> {

    private final MCCosmeticsManager cosmeticManager;
    
    public SelectionMenu(MCCosmeticsPlugin core, MenuManager manager, MCCosmeticsManager cmanager, String type) {
        super(core, manager, new MenuProp(core, "menus/selection_" + type.toLowerCase(), "Menu", null));

        this.cosmeticManager = cmanager;
    }

    @Override
    public void open(Player player, SelectionMenuContext context) {

    }

    @Override
    public EditableMenuBuilder<SelectionMenuContext> build(EditableMenuBuilder<SelectionMenuContext> builder) {
        builder.getIcon("BACK").ifPresent(icon -> {
            icon.getBuilder().click((context,player) -> {
                playMenuClick(player);
                getPlugin().getMenuManager().getCustomizeMenu().open(player, (Profile) context.getProfile());
            });
        });
        
        builder.getIcon("REMOVE_COSMETIC").ifPresent(icon -> {
            icon.getBuilder().click((context,player) -> {
                final var profile = context.getProfile();
                playMenuClick(player);
                cosmeticManager.unequip(profile);
                profile.unequip(cosmeticManager.getCosmeticClass());
                if(Property.Boolean(Scope.CONFIG, "Configuration.Menus.Close-On-Unequip", false).get()) {
                    player.closeInventory();
                }
                
                Schedulers.sync().runLater(() -> {
                    cosmeticManager.unequip(profile);
                }, 5);
            });
        });

        builder.getIcon("FILTER").ifPresent(icon -> {
            icon.getBuilder().click((context,player) -> {
                player.closeInventory();
                CommandHelper.sendEditorMessage(player,
                        "Type in the text you'd like to search for:",
                        "<gray>Type cancel to abort filtering.");

                ChatPrompt.listen(player, (input) -> {
                    if(input.equalsIgnoreCase("cancel")) {
                        return ChatPrompt.Response.ACCEPTED;
                    }

                    context.setFilter(input);

                    return ChatPrompt.Response.ACCEPTED;
                }).thenAcceptSync(in -> {
                    context.openMenu(player);
                });
            });
        });

        builder.getIcon("RESET_FILTER").ifPresent(icon -> {
            icon.getBuilder().click((context,player) -> {
                context.clearFilter();
                context.openMenu(player);
            });
        });

        builder.getIcon("SHOW_ALL").ifPresent(icon -> {
            icon.getBuilder().click((context,player) -> {
                context.setShowingUncollected(true);
                context.openMenu(player);

                builder.getIcon("SHOW_COLLECTED").ifPresent(invertedIcon -> {
                    icon.getSlots().forEach(slot -> {
                        final MenuHelper.OpenMenu<SelectionMenuContext> openMenu = (MenuHelper.OpenMenu<SelectionMenuContext>) MenuHelper.getOpenMenu(player);

                        if(openMenu.getMenu().getHolder() == null || !(openMenu.getMenu().getHolder() instanceof CosmeticMenu)) {
                            return;
                        }
                        Icon<SelectionMenuContext> newIcon = invertedIcon.getBuilder().build();
                        openMenu.setIcon(slot, newIcon);
                    });
                });
            });
        });

        builder.getIcon("SHOW_COLLECTED").ifPresent(icon -> {
            icon.getBuilder().click((context,player) -> {
                context.setShowingUncollected(false);
                context.openMenu(player);

                builder.getIcon("SHOW_ALL").ifPresent(invertedIcon -> {
                    icon.getSlots().forEach(slot -> {
                        final MenuHelper.OpenMenu<SelectionMenuContext> openMenu = (MenuHelper.OpenMenu<SelectionMenuContext>) MenuHelper.getOpenMenu(player);

                        if(openMenu.getMenu().getHolder() == null || !(openMenu.getMenu().getHolder() instanceof CosmeticMenu)) {
                            return;
                        }
                        Icon<SelectionMenuContext> newIcon = invertedIcon.getBuilder().build();
                        openMenu.setIcon(slot, newIcon);
                    });
                });
            });
        });

        buildAdvancedButtonOptions(builder);
        return builder;
    }

}
