package io.lumine.cosmetics.types;

import com.google.common.collect.Lists;
import com.google.common.collect.Maps;

import io.lumine.cosmetics.MCCosmeticsPlugin;
import io.lumine.cosmetics.api.cosmetics.Cosmetic;
import io.lumine.cosmetics.api.cosmetics.CosmeticVariant;
import io.lumine.cosmetics.api.cosmetics.EquippedCosmetic;
import io.lumine.cosmetics.api.menus.SelectionMenuContext;
import io.lumine.cosmetics.commands.CommandHelper;
import io.lumine.cosmetics.config.Scope;
import io.lumine.cosmetics.constants.Permissions;
import io.lumine.cosmetics.menus.CosmeticMenu;
import io.lumine.cosmetics.menus.SelectionMenuData;
import io.lumine.cosmetics.players.Profile;
import io.lumine.mythic.api.packs.Pack;
import io.lumine.mythic.bukkit.MythicBukkit;
import io.lumine.mythic.bukkit.utils.Schedulers;
import io.lumine.mythic.bukkit.utils.config.properties.Property;
import io.lumine.mythic.bukkit.utils.config.properties.types.*;
import io.lumine.mythic.bukkit.utils.items.ItemFactory;
import io.lumine.mythic.bukkit.utils.menu.Icon;
import io.lumine.mythic.bukkit.utils.menu.IconBuilder;
import io.lumine.mythic.bukkit.utils.serialize.Chroma;
import io.lumine.mythic.bukkit.utils.text.Text;
import io.papermc.paper.datacomponent.DataComponentTypes;
import lombok.Getter;
import org.bukkit.Material;
import org.bukkit.NamespacedKey;
import org.bukkit.inventory.ItemStack;

import java.io.File;
import java.util.List;
import java.util.Map;
import java.util.Optional;

public abstract class AbstractCosmetic extends Cosmetic {

    protected static final StringProp NAMESPACE = Property.String(Scope.NONE, "Namespace", null);
    protected static final StringProp PERMISSION = Property.String(Scope.NONE, "Permission", null);
	protected static final EnumProp<Material> MATERIAL = Property.Enum(Scope.NONE, Material.class, "Material", Material.EMERALD);
	protected static final StringProp MODEL = Property.String(Scope.NONE, "Model", null);
	protected static final LangProp DISPLAY = Property.Lang(Scope.NONE, "Display");
	protected static final LangListProp DESCRIPTION = Property.LangList(Scope.NONE, "Description");
	protected static final StringProp TEXTURE = Property.String(Scope.NONE, "SkullTexture");

    protected static final ColorProp COLOR = Property.Color(Scope.NONE, "Color");
    protected static final BooleanProp COLORABLE = Property.Boolean(Scope.NONE, "Colorable", false);
    protected static final StringListProp COLOR_SUGGESTIONS = Property.StringList(Scope.NONE, "ColorSuggesstions");

    protected static final NodeListProp VARIANTS = Property.NodeList(Scope.NONE, "Variants");

    private final MCCosmeticsManager manager;
    
	@Getter protected final Pack mythicPack;
	@Getter protected final File file;
    @Getter protected final String id;
	@Getter protected final String key;
	@Getter protected final String namespace;
	@Getter protected final String permission;
	@Getter protected final List<String> sources = Lists.newArrayList();

	// Menu Item
	protected Material material;
	@Getter protected String display;
	protected List<String> description;
    @Getter protected Chroma color;
	@Getter protected boolean colorable;
	@Getter protected List<Chroma> colorSuggestions = Lists.newArrayList();

	@Getter protected ItemStack menuItem;

	protected Map<String,CosmeticVariant> variants = Maps.newConcurrentMap();

	public AbstractCosmetic(MCCosmeticsManager manager, File file, String type, String key) {
		this(manager, null, file, type, key);
	}

	public AbstractCosmetic(MCCosmeticsManager manager, Pack pack, File file, String type, String key) {
		super(manager, type, key);
		this.manager = manager;
		this.mythicPack = pack;
		this.file = file;
		this.key = key;
		this.namespace = NAMESPACE.fget(file,this);
		
		if(namespace == null) {
		    this.id = key;
		} else {
		    this.id = namespace + "." + key;
		}
		
		String perm = PERMISSION.fget(file,this);
		
		if(perm == null) {
		    if(namespace == null) {
                this.permission = Permissions.COSMETIC_PERMISSION_PREFIX + "." + type.toLowerCase() + "." + key.toLowerCase();
		    } else {
		        this.permission = Permissions.COSMETIC_PERMISSION_PREFIX + "." + type.toLowerCase() + "." + namespace.toLowerCase() + "." + key.toLowerCase();
		    }
		} else {
		    this.permission = perm;
		}
		
		this.material = MATERIAL.fget(file,this);
		this.display = DISPLAY.fget(file,this);
		this.description = DESCRIPTION.fget(file,this);
        this.color = COLOR.fget(file,this);
		this.colorable = COLORABLE.fget(file,this);

		if(this.color != null) {
			this.colorSuggestions.add(this.color);
		}
		for(var line : COLOR_SUGGESTIONS.fget(file,this)) {
			final var chroma = Chroma.of(line);
			this.colorSuggestions.add(chroma);
		}

		for(String node : VARIANTS.fget(file,this)) {
		    var variant = new AbstractCosmeticVariant(this,file,node);
		    variants.put(variant.getKey(), variant);
		}
		
		if(material == Material.PLAYER_HEAD) {
			this.menuItem = ItemFactory.of(this.material)
					.name(Text.colorize(this.getDisplay()))
					.hideAttributes()
					.lore(description)
					//.skullTexture(TEXTURE.get(this))
					.build();
		} else {
			this.menuItem = ItemFactory.of(this.material)
					.name(Text.colorize(this.getDisplay()))
					.hideAttributes()
					.lore(description)
					.build();
		}

		var model = MODEL.fget(file,this);

		if(model != null) {
			NamespacedKey itemModelKey = null;

			if(MythicBukkit.inst().getCompatibility().getCrucible().isPresent()) {
				final var generationFeedback = MythicBukkit.inst().getCompatibility().getCrucible().get().addItemToGeneration(mythicPack, material, 0, model);

				if(generationFeedback.getItemModel().isPresent()) {
					itemModelKey = generationFeedback.getItemModel().get();
				}
			}
			if(itemModelKey == null) {
				if (!model.contains(":")) {
					model = "minecraft:" + model;
				}
				itemModelKey = NamespacedKey.fromString(model);
			}
			this.menuItem.setData(DataComponentTypes.ITEM_MODEL, itemModelKey);
		}
	}

	public Icon<SelectionMenuContext> getIcon() {
		return IconBuilder.<SelectionMenuContext>create()
				.name(this.getDisplay())
				.itemStack(this.menuItem)
				.hideFlags()
				.lore(context -> {
					List<String> desc = Lists.newArrayList(description);
					if(!context.getProfile().has(this)) {
						desc.add("");
						desc.add(Text.colorizeLegacy("<red>Not Unlocked"));
					}
					return desc; 
				})
				.click((context,player) -> {
					if(context.getProfile().getPlayer().isOp() || context.getProfile().has(this)) {
					    CosmeticMenu.playMenuClick(player);

						if(colorable) {
							MCCosmeticsPlugin.inst().getMenuManager().openColorPicker(manager, (Profile) context.getProfile(), (SelectionMenuData) context, this);
						} else {
							if (manager.getWardrobeManager().isInWardrobe(player)) {
								var mannequin = manager.getWardrobeManager().getMannequin(player);

								manager.equipMannequin(mannequin, new EquippedCosmetic(this));
								CommandHelper.sendSuccess(player, "Set wardrobe " + getType() + " to " + getDisplay());
							} else {
								if (MCCosmeticsPlugin.inst().getConfiguration().getEquipDelay() > 0) {
									Schedulers.sync().runLater(() -> {
										context.getProfile().equip(this);
									}, MCCosmeticsPlugin.inst().getConfiguration().getEquipDelay());
								} else {
									context.getProfile().equip(this);
								}
								CommandHelper.sendSuccess(player, "Set your " + getType() + " to " + getDisplay());
							}
							player.closeInventory();
						}
					} else {
						CommandHelper.sendError(player, Property.String(Scope.CONFIG,
								"Configuration.Language.Cosmetic-Not-Unlocked","You haven't unlocked that cosmetic!").get());
					}
				})
				.rightClick((context,player) -> {
                    if(context.getProfile().getPlayer().isOp() || context.getProfile().has(this)) {
                        
                    } else {
                        CommandHelper.sendError(player, Property.String(Scope.CONFIG,
                                "Configuration.Language.Cosmetic-Not-Unlocked","You haven't unlocked that cosmetic!").get());
                    }
                }).build();
	}

	public boolean hasVariants() {
	    return variants.isEmpty() == false;
	}
	
	public Optional<CosmeticVariant> getVariant(String id) {
	    return Optional.ofNullable(variants.getOrDefault(id, null));
	}

	@Override
	public String getPropertyNode() {
		return this.key;
	}

}
