package io.lumine.cosmetics.types.back;

import com.google.common.collect.Lists;
import io.lumine.cosmetics.MCCosmeticsPlugin;
import io.lumine.cosmetics.api.cosmetics.Cosmetic;
import io.lumine.cosmetics.api.cosmetics.EquippedCosmetic;
import io.lumine.cosmetics.api.cosmetics.ItemCosmetic;
import io.lumine.cosmetics.api.cosmetics.manager.HideableCosmetic;
import io.lumine.cosmetics.api.players.CosmeticProfile;
import io.lumine.cosmetics.api.players.wardrobe.Mannequin;
import io.lumine.cosmetics.logging.MCLogger;
import io.lumine.cosmetics.types.MCCosmeticsManager;
import io.lumine.cosmetics.types.gestures.Gesture;
import io.lumine.cosmetics.nms.cosmetic.VolatileAppliedCosmeticHelper;
import io.lumine.mythic.api.adapters.AbstractVector;
import io.lumine.mythic.api.packs.Pack;
import io.lumine.mythic.api.volatilecode.virtual.PacketArmorStand;
import io.lumine.mythic.api.volatilecode.virtual.PacketEntity;
import io.lumine.mythic.api.volatilecode.virtual.PacketEntityRenderer;
import io.lumine.mythic.api.volatilecode.virtual.PacketItemDisplay;
import io.lumine.mythic.bukkit.BukkitAdapter;
import io.lumine.mythic.bukkit.MythicBukkit;
import io.lumine.mythic.bukkit.utils.Events;
import io.lumine.mythic.bukkit.utils.Schedulers;

import java.io.File;
import java.util.Map;
import java.util.UUID;
import java.util.concurrent.ConcurrentHashMap;

import io.lumine.mythic.bukkit.utils.logging.Log;
import io.papermc.paper.event.player.PlayerTrackEntityEvent;
import lombok.Data;
import org.bukkit.GameMode;
import org.bukkit.entity.EntityType;
import org.bukkit.entity.Player;
import org.bukkit.event.player.PlayerChangedWorldEvent;
import org.bukkit.event.player.PlayerGameModeChangeEvent;
import org.bukkit.event.player.PlayerMoveEvent;
import org.bukkit.event.player.PlayerTeleportEvent;
import org.bukkit.util.EulerAngle;

public class BackManager extends MCCosmeticsManager<BackAccessory> implements HideableCosmetic {

    private final Map<Player, PacketItemDisplay> renderedBacks = new ConcurrentHashMap<>();

    public BackManager(MCCosmeticsPlugin plugin) {
        super(plugin, BackAccessory.class);

        load(plugin);
    }
    
    public void load(MCCosmeticsPlugin plugin) {
        super.load(plugin);
        
        Events.subscribe(PlayerChangedWorldEvent.class)
            .handler(event -> {
                final var player = event.getPlayer();
                final var profile = plugin.getProfiles().getProfile(player);
                
                if(profile != null && profile.getEquipped(BackAccessory.class).isPresent()) {
                    unequip(profile);
                    
                    Schedulers.sync().runLater(() -> {
                        equip(profile);
                    }, 5);
                }
            })
            .bindWith(this);
        
        Events.subscribe(PlayerTeleportEvent.class)
            .handler(event -> {
                final var player = event.getPlayer();
                final var profile = plugin.getProfiles().getProfile(player);
                
                if(profile != null && profile.getEquipped(BackAccessory.class).isPresent()) {
                    unequip(profile);
                    
                    Schedulers.sync().runLater(() -> {
                        equip(profile);
                    }, 5);
                }
            })
            .bindWith(this);
        
        Events.subscribe(PlayerGameModeChangeEvent.class)
            .handler(event -> {
                if(event.getNewGameMode() == GameMode.SPECTATOR) {
                    final var player = event.getPlayer();
                    final var profile = plugin.getProfiles().getProfile(player);
                    
                    unequip(profile);
                } else if(event.getPlayer().getPlayer().getGameMode() == GameMode.SPECTATOR) {
                    final var player = event.getPlayer();
                    final var profile = plugin.getProfiles().getProfile(player);
                    
                    Schedulers.sync().runLater(() -> {
                        if(player.getGameMode() != GameMode.SPECTATOR) {
                            equip(profile);
                        }
                    }, 5);
                }
            })
            .bindWith(this);

        Schedulers.async().runRepeating(this::handleUpdate, 1, 1).bindWith(this);

        //Events.subscribe(PlayerMoveEvent.class)
        //    .handler(this::handlePlayerMove)
        //    .bindWith(this);

        Events.subscribe(PlayerTrackEntityEvent.class)
            .handler(this::handlePlayerTrack)
            .bindWith(this);
    }

    @Override
    public BackAccessory build(Pack pack, File file, String node) {
        return new BackAccessory(this, pack, file, node);
    }

    @Override
    public void equip(CosmeticProfile profile) {
        var maybeEquipped = profile.getEquipped(BackAccessory.class);
        if(maybeEquipped.isEmpty()) {
            return;
        }
        var equipped = maybeEquipped.get();
        var cosmetic = equipped.getCosmetic();

        if (!(cosmetic instanceof BackAccessory back)) {
            return;
        }
        final var itemStack = back.getCosmetic(equipped);
        final var aPlayer = BukkitAdapter.adapt(profile.getPlayer());

        if(back.getRendererType() == BackRendererType.DISPLAY_ENTITY) {
            final var packetDisplay = PacketItemDisplay.create()
                    .item(itemStack)
                    .interpolationDelay(0)
                    .interpolationDuration(1)
                    .yOffset(1.26F)
                    .height(0F)
                    .width(0F)
                    .scale(new AbstractVector(0.5, 0.5, 0.5))
                    .build(aPlayer.getLocation());

            packetDisplay.getRenderer().spawn(() -> {
                final var list = Lists.newArrayList(MythicBukkit.inst().getVolatileCodeHandler().getEntityHandler().getTrackedPlayers(aPlayer));
                list.add(aPlayer);
                return list;
            });
            packetDisplay.getRenderer().mountEntity(aPlayer);
            //Schedulers.async().runLater(() -> {
            //            packetDisplay.setTranslationTransformation(0, 1.26F, 0);
            //         }, 20);

            renderedBacks.put(profile.getPlayer(), packetDisplay);
        } else {
            getHelper().apply(profile);

            /*
            Log.info("Spawn ARMOR STAND Backpack");
            final var packetDisplay = PacketArmorStand.create()
                    .headItem(itemStack)
                    .build(aPlayer.getLocation());

            packetDisplay.getRenderer().spawn(() -> {
                final var list = MythicBukkit.inst().getVolatileCodeHandler().getEntityHandler().getTrackedPlayers(aPlayer);
                list.add(aPlayer);
                return list;
            });
            packetDisplay.getRenderer().mountEntity(aPlayer);

            renderedBacks.put(profile.getPlayer(), packetDisplay);

            /*
            packetDisplay.getRenderer().spawnWithMount(() -> {
                final var list = MythicBukkit.inst().getVolatileCodeHandler().getEntityHandler().getTrackedPlayers(aPlayer);
                list.add(aPlayer);
                return list;
            }, aPlayer);

             */
        }
    }

    @Override
    public void unequip(CosmeticProfile profile) {
        final var removed = renderedBacks.remove(profile.getPlayer());
        if(removed != null) {
            removed.destroy();
        } else {
            getHelper().unapply(profile);
        }
    }

    private void handleUpdate() {
        renderedBacks.entrySet().forEach(entry -> {
            final var player = entry.getKey();
            final var backpack = entry.getValue();

            float bodyYaw = player.getBodyYaw();
            float locationYaw = player.getLocation().getYaw();

            bodyYaw = (bodyYaw + 360) % 360;
            if (bodyYaw > 180) bodyYaw -= 360;

            locationYaw = (locationYaw + 360) % 360;
            if (locationYaw > 180) locationYaw -= 360;

            float diff = Math.abs(bodyYaw - locationYaw);

            if (diff > 180) {
                if (bodyYaw > locationYaw) {
                    locationYaw += 360;
                } else {
                    bodyYaw += 360;
                }
            }

            float yaw = (bodyYaw + locationYaw) / 2;

            yaw = (yaw + 360) % 360;
            if (yaw > 180) yaw -= 360;

            backpack.setInterpolationDuration(1);
            backpack.setRotation(yaw + 180, 0);
            backpack.update();
        });
    }

    private void handlePlayerMove(PlayerMoveEvent event) {
        final var player = event.getPlayer();
        final var backpack = renderedBacks.get(player);

        if(backpack != null && backpack instanceof PacketItemDisplay<?> itemDisplay) {
            final var yaw = event.getTo().getYaw();

            MCLogger.log("yaw {0}", yaw);
            itemDisplay.setRotation(yaw, 0);
            itemDisplay.update();
        }
    }

    private void handlePlayerTrack(PlayerTrackEntityEvent event) {
        if(!(event.getEntity() instanceof Player player)) {
            return;
        }
        final var backpack = renderedBacks.get(player);

        if(backpack != null) {
            backpack.getRenderer().updateRenderedPlayers();
        }
    }
    
    @Override
    public void equipMannequin(Mannequin mannequin, EquippedCosmetic cosmetic) {
        getHelper().equipMannequin(mannequin, cosmetic);
    }
    
    @Override
    public void unequipMannequin(Mannequin mannequin) {
        getHelper().unequipMannequin(mannequin);
    }

    @Override
    public void hide(CosmeticProfile profile, Cosmetic request) {
        if(!(request instanceof Gesture)) {
            return;
        }

        profile.setHidden(getCosmeticClass(), true);
        getHelper().unapply(profile);
    }

    @Override
    public void show(CosmeticProfile profile) {
        if(profile.getPlayer().isDead()) {
            return;
        }

        profile.setHidden(getCosmeticClass(), false);
        getHelper().apply(profile);
    }

    protected VolatileAppliedCosmeticHelper getHelper() {
        return (VolatileAppliedCosmeticHelper) getNMSHelper();
    }

}
