package io.lumine.cosmetics.types.chat;

import io.lumine.cosmetics.api.menus.SelectionMenuContext;
import io.lumine.cosmetics.api.players.CosmeticProfile;
import io.lumine.cosmetics.config.Scope;
import io.lumine.cosmetics.constants.CosmeticType;
import io.lumine.cosmetics.logging.MCLogger;
import io.lumine.cosmetics.types.AbstractCosmetic;
import io.lumine.mythic.api.packs.Pack;
import io.lumine.mythic.api.skills.placeholders.PlaceholderString;
import io.lumine.mythic.bukkit.MythicBukkit;
import io.lumine.mythic.bukkit.compatibility.CrucibleSupport;
import io.lumine.mythic.bukkit.utils.config.properties.Property;
import io.lumine.mythic.bukkit.utils.config.properties.types.IntProp;
import io.lumine.mythic.bukkit.utils.config.properties.types.StringProp;
import io.lumine.mythic.bukkit.utils.menu.Icon;
import lombok.Getter;

import java.io.File;

public class Sticker extends AbstractCosmetic {

	private static final StringProp TAG = Property.String(Scope.NONE, "Sticker", null);

	private static final StringProp MODEL = Property.String(Scope.NONE, "Model", null);

	private static final StringProp KEY = Property.String(Scope.NONE, "Key", null);
	private static final StringProp TYPE = Property.String(Scope.NONE, "Type", "bitmap");
	private static final StringProp FILE = Property.String(Scope.NONE, "Texture", null);
	private static final IntProp ASCENT = Property.Int(Scope.NONE, "Ascent", -1);
	private static final IntProp HEIGHT = Property.Int(Scope.NONE, "Height", -1);
	private static final StringProp CHAR = Property.String(Scope.NONE, "Char", null);

	@Getter private PlaceholderString chatTag;
	@Getter private String texturePath;

	public Sticker(StickerManager manager, Pack pack, File file, String key) {
	    super(manager, pack, file, CosmeticType.type(Sticker.class), key);

		this.chatTag = PlaceholderString.of(TAG.fget(file, this));
		if(this.chatTag == null) {
			this.texturePath = FILE.fget(file, this);
			if(this.texturePath == null) {
				this.texturePath = MODEL.fget(file, this);
			}
			if(this.texturePath == null) {
				MCLogger.error("Nothing defined for sticker {0}", key);
				return;
			}
			if(MythicBukkit.inst().getCompatibility().getCrucible().isEmpty()) {
				MCLogger.error("Error with Sticker {0} - Texture generation requires Crucible", key);
				return;
			}

			final var fontDef = CrucibleSupport.FontGenerationDef.builder()
					.key(KEY.fget(file, this))
					.type(TYPE.fget(file, this))
					.path(this.texturePath)
					.ascent(ASCENT.fget(file, this))
					.height(HEIGHT.fget(file, this))
					.character(CHAR.fget(file, this))
					.build();

			final var maybeChar = MythicBukkit.inst().getCompatibility().getCrucible().get().addFontImageToGeneration(pack, fontDef);
			if(maybeChar.isPresent()) {
				this.chatTag = PlaceholderString.of(maybeChar.get().toString());
			} else {
				MCLogger.error("Error with Sticker {0} - Texture generation failed", key);
				return;
			}
		}
	}

	public String getTexture() {
		return TEXTURE.fget(file, this);
	}

}
