package io.lumine.cosmetics.types.chat;

import io.lumine.cosmetics.MCCosmeticsPlugin;
import io.lumine.cosmetics.api.players.CosmeticProfile;
import io.lumine.cosmetics.config.Scope;
import io.lumine.cosmetics.types.MCCosmeticsManager;
import io.lumine.cosmetics.types.sprays.Spray;
import io.lumine.cosmetics.types.sprays.types.MapSpray;
import io.lumine.mythic.api.packs.Pack;
import io.lumine.mythic.bukkit.utils.Events;
import io.lumine.mythic.bukkit.utils.config.properties.Property;
import io.lumine.mythic.bukkit.utils.config.properties.types.BooleanProp;
import io.lumine.mythic.bukkit.utils.logging.Log;
import io.papermc.paper.event.player.AsyncChatEvent;
import net.kyori.adventure.text.Component;
import net.kyori.adventure.text.TextComponent;
import net.kyori.adventure.text.TextReplacementConfig;
import org.bukkit.event.EventPriority;

import java.io.File;
import java.util.regex.Pattern;

public class StickerManager extends MCCosmeticsManager<Sticker> {

    private static final BooleanProp EMOJIS = Property.Boolean(Scope.CONFIG, "Configuration.Stickers.UsableAsEmojis", false);
    private static final BooleanProp SPRAYS = Property.Boolean(Scope.CONFIG, "Configuration.Stickers.UsableAsSprays", false);

    private static final Pattern EMOJI_PATTERN = Pattern.compile(":([a-zA-Z0-9_]+):");

    public StickerManager(MCCosmeticsPlugin plugin) {
        super(plugin, Sticker.class);
        
        load(plugin);
    }

    @Override
    public void load(MCCosmeticsPlugin plugin) {
        super.load(plugin);

        if(EMOJIS.get()) {
            Events.subscribe(AsyncChatEvent.class, EventPriority.LOWEST).handler(event -> {
                final var player = event.getPlayer();
                final var profile = getProfiles().getProfile(event.getPlayer());
                var messageComponent = event.message();

                final var emojiReplacement = TextReplacementConfig.builder()
                        .match(EMOJI_PATTERN)
                        .replacement(matchResult -> {
                            String emojiKey = matchResult.content().substring(1, matchResult.content().length() - 1);

                            final var maybeSticker = getCosmetic(emojiKey);
                            if(maybeSticker.isEmpty()) {
                                return matchResult.build();
                            }
                            final var sticker = maybeSticker.get();

                            if(!profile.has(sticker)) {
                                return matchResult.build();
                            }

                            return Component.text().resetStyle()
                                    .append(
                                            Component.text(sticker.getChatTag().get()))
                                                    .hoverEvent(Component.text(sticker.getDisplay())).build();
                        })
                        .build();

                messageComponent = messageComponent.replaceText(emojiReplacement);

                event.message(messageComponent);
            }).bindWith(this);
        }

        if(SPRAYS.get()) {
            final var sprayManager = plugin.getSprayManager();

            for(var sticker : this.getAllCosmetics()) {
                final var spray = new MapSpray(sprayManager, sticker);
                sprayManager.registerCosmetic(spray);
            }
        }
    }

    @Override
    public Sticker build(Pack pack, File file, String node) {
        return new Sticker(this, pack, file, node);
    }

    @Override
    public void equip(CosmeticProfile profile) {}

    @Override
    public void unequip(CosmeticProfile profile) {}
    
}
