package io.lumine.cosmetics.types.effects;

import io.lumine.cosmetics.MCCosmeticsPlugin;
import io.lumine.cosmetics.api.cosmetics.Cosmetic;
import io.lumine.cosmetics.api.cosmetics.EquippedCosmetic;
import io.lumine.cosmetics.api.cosmetics.manager.HideableCosmetic;
import io.lumine.cosmetics.api.players.CosmeticProfile;
import io.lumine.cosmetics.api.players.wardrobe.Mannequin;
import io.lumine.cosmetics.config.Scope;
import io.lumine.cosmetics.nms.cosmetic.VolatileEquipmentCosmeticHelper;
import io.lumine.cosmetics.players.Profile;
import io.lumine.cosmetics.types.MCCosmeticsManager;
import io.lumine.cosmetics.types.equipment.CosmeticEquipmentSlot;
import io.lumine.cosmetics.types.gestures.Gesture;
import io.lumine.mythic.api.adapters.AbstractEntity;
import io.lumine.mythic.api.adapters.AbstractLocation;
import io.lumine.mythic.api.packs.Pack;
import io.lumine.mythic.api.skills.SkillCaster;
import io.lumine.mythic.api.skills.SkillTrigger;
import io.lumine.mythic.bukkit.BukkitAdapter;
import io.lumine.mythic.bukkit.MythicBukkit;
import io.lumine.mythic.bukkit.utils.Schedulers;
import io.lumine.mythic.bukkit.utils.config.properties.Property;
import io.lumine.mythic.bukkit.utils.config.properties.types.IntProp;
import io.lumine.mythic.bukkit.utils.tasks.Task;
import io.lumine.mythic.core.skills.SkillTriggers;

import java.io.File;
import java.util.Map;
import java.util.concurrent.ConcurrentHashMap;
import java.util.function.Consumer;

public class EffectManager extends MCCosmeticsManager<Effect> implements HideableCosmetic, Consumer<Task> {

    private static final IntProp EFFECT_TIMER = Property.Int(Scope.CONFIG, "Configuration.Effects.ClockInterval", 1);

    private final Map<CosmeticProfile,Effect> effects = new ConcurrentHashMap<>();

    public EffectManager(MCCosmeticsPlugin plugin) {
        super(plugin, Effect.class);
        
        load(plugin);
    }

    @Override
    public void load(MCCosmeticsPlugin plugin) {
        super.load(plugin);

        final long interval = EFFECT_TIMER.get();
        Schedulers.async().runRepeating(this, interval, interval).bindWith(this);
    }

    @Override
    public Effect build(Pack pack, File file, String node) {
        return new Effect(this, pack, file, node);
    }

    @Override
    public void equip(CosmeticProfile profile) {
        final var effect = (Effect) profile.getEquipped(Effect.class).get().getCosmetic();
        this.effects.put(profile, effect);
    }

    @Override
    public void unequip(CosmeticProfile profile) {
        this.effects.remove(profile);
    }
    
    @Override
    public void equipMannequin(Mannequin mannequin, EquippedCosmetic cosmetic) {

    }
    
    @Override
    public void unequipMannequin(Mannequin mannequin) {

    }

    @Override
    public void show(CosmeticProfile profile) {
        profile.setHidden(getCosmeticClass(), false);
    }

    @Override
    public void hide(CosmeticProfile profile, Cosmetic request) {
        if(!(request instanceof Gesture)) {
            return;
        }
        profile.setHidden(getCosmeticClass(), true);
    }

    @Override
    public void accept(Task task) {
        final var interval = task.getTimesRan();

        for(var entry : effects.entrySet()) {
            if(interval % entry.getValue().getInterval() == 0) {
                final var profile = entry.getKey();
                final var effect = entry.getValue();

                final var caster = MythicBukkit.inst().getSkillManager().getCaster(BukkitAdapter.adapt(profile.getPlayer()));
                final var meta = MythicBukkit.inst().getSkillManager().getEventBus().buildSkillMetadata(SkillTriggers.TIMER, caster, caster.getEntity(), caster.getLocation(), false);
                effect.getSkill().execute(meta);
            }
        }
    }
}
