package io.lumine.cosmetics.types.sprays.types;

import io.lumine.cosmetics.config.MCCLang;
import io.lumine.cosmetics.config.Scope;
import io.lumine.cosmetics.logging.MCLogger;
import io.lumine.cosmetics.types.chat.Sticker;
import io.lumine.cosmetics.types.sprays.Spray;
import io.lumine.cosmetics.types.sprays.SprayImage;
import io.lumine.cosmetics.types.sprays.SprayManager;
import io.lumine.mythic.api.packs.Pack;
import io.lumine.mythic.bukkit.utils.config.properties.Property;
import io.lumine.mythic.bukkit.utils.config.properties.types.StringProp;
import lombok.Getter;
import org.bukkit.entity.Player;

import java.io.File;

public class MapSpray extends Spray {

    private static final StringProp MODEL = Property.String(Scope.NONE, "Model", null);
    private static final StringProp IMAGE = Property.String(Scope.NONE, "Texture", null);

    @Getter private String imagePath;
    @Getter private SprayImage image;

    public MapSpray(SprayManager manager, Pack pack, File file, String key) {
        super(manager, pack, file, key);

        this.imagePath = IMAGE.fget(file, this);
        if(imagePath == null) {
            imagePath = MODEL.fget(file, this);
        }

        if(this.imagePath == null) {
            MCLogger.error(MCCLang.get("cosmetic.spray.error.no-image-specified", "No image specified for spray {0}", key));
            return;
        }

        this.image = getSprayImage(imagePath);

        if(this.image == null) {
            MCLogger.error(MCCLang.get("cosmetic.spray.error.image-not-found", "Image {0} for spray {1} was not found", imagePath, key));
        }
    }

    public MapSpray(SprayManager manager, Sticker sticker) {
        super(manager, sticker.getMythicPack(), sticker.getFile(), sticker.getKey());

        this.imagePath = sticker.getTexturePath();

        if(this.imagePath == null) {
            MCLogger.error(MCCLang.get("cosmetic.spray.error.no-image-specified", "No image specified for spray {0}", key));
            return;
        }

        this.image = getSprayImage(imagePath);

        if(this.image == null) {
            MCLogger.error(MCCLang.get("cosmetic.spray.error.image-not-found", "Image {0} for spray {1} was not found", imagePath, key));
        }
    }

    public void use(Player player) {
        if(this.image == null) {
            MCLogger.error(MCCLang.get("cosmetic.spray.error.image-not-found", "Image {0} for spray {1} was not found", imagePath, key));
            return;
        }
        ((SprayManager) getManager()).useSpray(player, this);
    }

    protected SprayImage getSprayImage(String key) {
        final var assetsFolder = mythicPack.getPackFolder("assets");
        final var texturesFolder = new File(assetsFolder, "textures");

        final var sprayFilePng = new File(texturesFolder, key + ".png");
        final var sprayFileJpg = new File(texturesFolder, key + ".jpg");

        if (sprayFilePng.exists()) {
            return createSprayImage(sprayFilePng);
        } else if (sprayFileJpg.exists()) {
            return createSprayImage(sprayFileJpg);
        }
        return null;
    }

    private SprayImage createSprayImage(File file) {
        final var manager = ((SprayManager) getManager());
        return new SprayImage(manager, file, manager.getNextMapId());
    }

}
